@@ -1,7 +1,7 @@
 package Module::Patch;
 
 our $DATE = '2018-10-07'; # DATE
-our $VERSION = '0.273'; # VERSION
+our $VERSION = '0.274'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
@@ -60,7 +60,11 @@ sub import {
         # patch already applied, ignore
         return if ${"$self\::handles"};
 
-        my $pdata = $self->patch_data or
+        unless (${"$self\::patch_data_cached"}) {
+            ${"$self\::patch_data_cached"} = $self->patch_data;
+        }
+
+        my $pdata = ${"$self\::patch_data_cached"} or
             die "BUG: $self: No patch data supplied";
         my $v = $pdata->{v} // 1;
         my $curv = 3;
@@ -150,14 +154,12 @@ sub import {
 }
 
 sub unimport {
-    no strict 'refs';
-
     my $self = shift;
 
     if ($self eq __PACKAGE__) {
         # we are not subclassed, do nothing
     } else {
-        my $pdata = $self->patch_data or
+        my $pdata = ${"$self\::patch_data_cached"} or
             die "BUG: $self: No patch data supplied";
 
         if ($pdata->{before_unpatch}) {
@@ -181,8 +183,6 @@ sub patch_data {
 }
 
 sub patch_package {
-    no strict 'refs';
-
     my ($package0, $patches_spec, $opts) = @_;
     $opts //= {};
 
@@ -191,13 +191,7 @@ sub patch_package {
         die "FATAL: Target module '$target' not loaded"
             unless package_exists($target);
         my $target_version = ${"$target\::VERSION"};
-        my @target_subs;
-        my %tp = list_package_contents($target);
-        for (keys %tp) {
-            if (ref($tp{$_}) eq 'CODE' && !/^\*/) {
-                push @target_subs, $_;
-            }
-        }
+        my $target_subs;
 
         my $i = 0;
       PATCH:
@@ -227,7 +221,16 @@ sub patch_package {
             my @s;
             for my $sub_name (@$sub_names) {
                 if (ref($sub_name) eq 'Regexp') {
-                    for (@target_subs) {
+                    unless ($target_subs) {
+                        $target_subs = [];
+                        my %tp = list_package_contents($target);
+                        for (keys %tp) {
+                            if (ref($tp{$_}) eq 'CODE' && !/^\*/) {
+                                push @$target_subs, $_;
+                            }
+                        }
+                    }
+                    for (@$target_subs) {
                         push @s, $_ if $_ !~~ @s && $_ =~ $sub_name;
                     }
                 } else {
@@ -288,7 +291,7 @@ Module::Patch - Patch package with a set of patches
 
 =head1 VERSION
 
-This document describes version 0.273 of Module::Patch (from Perl distribution Module-Patch), released on 2018-10-07.
+This document describes version 0.274 of Module::Patch (from Perl distribution Module-Patch), released on 2018-10-07.
 
 =head1 SYNOPSIS
 
